package ACL2::Proceso::Fabrica;

=head1 NAME

ACL2::Proceso::Fabrica - fE<aacute>brica de ACL2::Proceso con registro de identificadores

=head1 FUNCTIONS

=cut

#
#    Fábrica de Procesos 
#    Copyright (C) 2006-2008 Antonio García Domínguez
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use ACL2::Localizacion;

my $lh = ACL2::Localizacion->get_handle;

# Esta fábrica realmente no sabe cómo crear los objetos: se limita a
# ser un punto central de registro de funciones de creación.

my %reg_identificadores = ();
my $paquete_base = join '', __PACKAGE__ =~ /(.*)::.*/;

# Todo Proceso que desee ser creado, debe pues registrar una
# referencia a un mE<eacute>todo que devuelva los parámetros a pasar al mE<eacute>todo
# estático new o undef si no la salida del mE<eacute>todo no se corresponde
# con la esperada.

=head1 FUNCTIONS

Las funciones que aporta este mE<oacute>dulo son:

=head2 crear

Uso: ME<eacute>todo estE<aacute>tico pE<uacute>blico.

Responsabilidades: Crea un nuevo Proceso de entre aquellos que tiene registrados.

ParE<aacute>metros:

=over 2

=item * 

salida: salida de la submeta.

=back

Devuelve: objeto de la subclase pertinente de ACL2::Proceso.

Fallos: Fallo fatal si no ha podido crear ningE<uacute>n objeto.

=cut

sub crear
{
    my ($paquete, $salida) = @_;

    foreach my $clase (keys %reg_identificadores) {
	my $rh_args = $reg_identificadores{$clase}->($salida);
	return new $clase ($rh_args) if defined($rh_args);
    }

    die $lh->maketext("No se conoce el procedimiento seguido.");
}


=head2 registrar

Uso: ME<eacute>todo estE<aacute>tico de paquete.

Responsabilidades: Registra una nueva subrutina de
identificaciE<oacute>n.

ParE<aacute>metros:

=over 2

=item *

clase: cadena con nombre de la clase del objeto cuyo constructor ha
de invocarse.

=item *

subrutina: referencia a una subrutina que devuelve undef si la
salida no es la esperada para dicha clase de proceso o una lista con
los argumentos para new de lo contrario.

=back

Devuelve: nada.

=cut

sub registrar
{
    my ($paquete, $clase, $subrutina) = @_;

    # Paquete del registrante
    my $paquete_invocador = join '', (caller(1))[3] =~ /(.*)::.*/;

    die $lh->maketext("Acceso a [_1] restringido a miembros del paquete [_2]", "registrar", $paquete_base)
	if $paquete_invocador !~ /^${paquete_base}::[a-zA-Z_0-9]*/;
    die $lh->maketext("Ya se ha registrado la clase [_1]", $clase) 
	if exists $reg_identificadores{$clase};

    $reg_identificadores{$clase} = $subrutina;    
}

1;
